/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	The implementation of our dialogs.

****************************************************************************/

#include "gba.h"

Bitmap g_dialog;
Bitmap g_background;

#define TEXT_X				(20)
#define TEXT_Y				(g_dialog.GetHeight() * 3 / 4)
#define TEXT_WIDTH			(g_dialog.GetWidth() - 40)
#define TEXT_HEIGHT			(TextGetFontHeight(g_font))

#define CAPTION_X			(20)
#define CAPTION_Y			(20)

#define USER_INPUT			TEXT_COLOR_GREEN
#define CAPTION_COLOR		TEXT_COLOR_RED

FString g_userString;
static u8 s_dirty;
void (*s_stringGetCallback)(void);

static TString s_dialogFile("dlg_eng.bmp");

static u8 s_mode;
static real s_delayTime;
static real s_delayTimestamp;

enum {
	MODE_RECEIVE_STRING,
	MODE_RECEIVE_ACK,
};

static TString s_okToContinue("Press <Enter> or <Esc> to continue.");

/////////////////////////////////////////////////////////////////////////////
//  Pop up a message box for the user, and wait for him to press enter or
// escape.  We have the option of popping up an "annoyance" message that has
// a delay imposed.
void DialogDeliverMessage(const FString & line1,
						  const FString & line2,
						  real delayTime) {
	Rect region;
	u16 dlgX, dlgY;
	RenderOptions ro;
	FString f;

	SetMode(MODE_QUERY_USER);

	// Let's prepare the dialog window.
	if (!(g_dialog.GetHeight())) {
		IString i;
		i = s_dialogFile;
		g_dialog.Import(i);
	}

	dlgX = (g_monitor.GetWidth() - g_dialog.GetWidth()) / 2;
	dlgY = (g_monitor.GetHeight() - g_dialog.GetHeight()) / 2;

	// Clear the first line.
	region.x = CAPTION_X;
	region.y = CAPTION_Y;
	region.w = TEXT_WIDTH;
	region.h = TEXT_HEIGHT;
	g_dialog.Clear(0, &region);

	// Clear the second line.
	region.x = CAPTION_X;
	region.y = CAPTION_Y + TEXT_HEIGHT;
	region.w = TEXT_WIDTH;
	region.h = TEXT_HEIGHT;
	g_dialog.Clear(0, &region);

	// Clear the text section.
	region.x = TEXT_X;
	region.y = TEXT_Y;
	region.w = TEXT_WIDTH;
	region.h = TEXT_HEIGHT;
	g_dialog.Clear(0, &region);

	g_background.ExtractFrom(g_monitor,
							 dlgX,
							 dlgY,
							 g_dialog.GetWidth(),
							 g_dialog.GetHeight());
	f = CAPTION_COLOR;
	f += line1;
	TextWrite(g_dialog,
			  g_font,
			  CAPTION_X,
			  CAPTION_Y,
			  0,
			  f);

	f = CAPTION_COLOR;
	f += line2;
	TextWrite(g_dialog,
			  g_font,
			  CAPTION_X,
			  CAPTION_Y + TEXT_HEIGHT,
			  0,
			  f);

	f = CAPTION_COLOR;
	f += s_okToContinue;
	TextWrite(g_dialog,
			  g_font,
			  TEXT_X,
			  TEXT_Y,
			  0,
			  f);

	// The initial drawing of the dialog.
	ro.blend = &g_background;
	g_dialog.Render(g_monitor,
					dlgX,
					dlgY,
					ro);
	g_monitor.Refresh();
	g_dialog.Render(g_monitor,
					dlgX,
					dlgY,
					ro);
	g_monitor.Refresh();
	g_userString = "";
	if (delayTime) {
		s_delayTime = delayTime;
		s_delayTimestamp = GetCPUSeconds();
	}
	s_mode = MODE_RECEIVE_ACK;
	MouseDisplay(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
//  Get a string from the user.
void DialogGetString(const FString & caption,
					 void (*Callback)(void)) {
	Rect region;
	u16 dlgX, dlgY;
	RenderOptions ro;
	FString f;

	SetMode(MODE_QUERY_USER);

	// Let's prepare the dialog window.
	if (!(g_dialog.GetHeight())) {
		IString i;
		i = s_dialogFile;
		g_dialog.Import(i);
	}

	dlgX = (g_monitor.GetWidth() - g_dialog.GetWidth()) / 2;
	dlgY = (g_monitor.GetHeight() - g_dialog.GetHeight()) / 2;

	// Clear the caption section.
	region.x = CAPTION_X;
	region.y = CAPTION_Y;
	region.w = TEXT_WIDTH;
	region.h = TEXT_HEIGHT;
	g_dialog.Clear(0, &region);

	// Clear the text section.
	region.x = TEXT_X;
	region.y = TEXT_Y;
	region.w = TEXT_WIDTH;
	region.h = TEXT_HEIGHT;
	g_dialog.Clear(0, &region);

	// Clear the second line.
	region.x = CAPTION_X;
	region.y = CAPTION_Y + TEXT_HEIGHT;
	region.w = TEXT_WIDTH;
	region.h = TEXT_HEIGHT;
	g_dialog.Clear(0, &region);

	g_dialog.Clear(0, &region);
	g_background.ExtractFrom(g_monitor,
							 dlgX,
							 dlgY,
							 g_dialog.GetWidth(),
							 g_dialog.GetHeight());
	f = CAPTION_COLOR;
	f += caption;
	TextWrite(g_dialog,
			  g_font,
			  CAPTION_X,
			  CAPTION_Y,
			  0,
			  f);

	// The initial drawing of the dialog.
	ro.options = Bitmap::TRANSLUCENT;
	ro.blend = &g_background;
	g_dialog.Render(g_monitor,
					dlgX,
					dlgY,
					ro);
	g_monitor.Refresh();
	g_dialog.Render(g_monitor,
					dlgX,
					dlgY,
					ro);
	g_monitor.Refresh();
	g_userString = USER_INPUT;
	s_stringGetCallback = Callback;
	s_mode = MODE_RECEIVE_STRING;
	MouseDisplay(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
//  Receive user input.
void DialogProcessInput(Key key) {
	RenderOptions ro;
	u16 dlgX, dlgY;

	if (s_delayTime > 0) {
		if (GetCPUSeconds() - s_delayTimestamp < s_delayTime) {
			return;
		}
		s_delayTime = 0;
	}
	key &= KB_BASE_MASK;

	dlgX = (g_monitor.GetWidth() - g_dialog.GetWidth()) / 2;
	dlgY = (g_monitor.GetHeight() - g_dialog.GetHeight()) / 2;
	switch(key) {
		case KB_ESC:
			SetMode(MODE_DEBUGGER);
			g_userString = "";
			g_monitor.Refresh();
			g_background.Render(g_monitor,
								dlgX,
								dlgY,
								ro);
			g_monitor.Refresh();
			g_background.Render(g_monitor,
								dlgX,
								dlgY,
								ro);
			g_monitor.Refresh();
			MouseDisplay(TRUE);
			break;
		case KB_ENTER:
			SetMode(MODE_DEBUGGER);
			g_monitor.Refresh();
			g_background.Render(g_monitor,
								dlgX,
								dlgY,
								ro);
			g_monitor.Refresh();
			g_background.Render(g_monitor,
								dlgX,
								dlgY,
								ro);
			g_monitor.Refresh();
			MouseDisplay(TRUE);
			if (s_mode == MODE_RECEIVE_STRING) {
				s_stringGetCallback();
			}
			break;
		default:
			if (s_mode == MODE_RECEIVE_STRING) {
				Rect region;
	
				if (key == KB_BACK) {
					g_userString.Chop();
					if (!g_userString.Length()) {
						g_userString = USER_INPUT;
					}
				} else {
					g_userString += KeyToChar(key);
				}
				region.x = TEXT_X;
				region.y = TEXT_Y;
				region.w = TEXT_WIDTH;
				region.h = TEXT_HEIGHT;
				g_dialog.Clear(0, &region);
				TextWrite(g_dialog,
						  g_font,
						  TEXT_X,
						  TEXT_Y,
						  0,
						  g_userString);
				s_dirty = 0xff;
			}
			break;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Render the dialog.
void DialogRender(void) {
	u8 b = 1 << g_monitor.GetCurrentBuffer();
	u16 dlgX, dlgY;
	u16 x, y;
	color c;
	u16 fw;

	dlgX = (g_monitor.GetWidth() - g_dialog.GetWidth()) / 2;
	dlgY = (g_monitor.GetHeight() - g_dialog.GetHeight()) / 2;
	if (b & s_dirty) {
		RenderOptions ro;

		if (s_mode == MODE_RECEIVE_STRING) {
			ro.options = Bitmap::TRANSLUCENT;
		}
		ro.blend = &g_background;
		g_dialog.Render(g_monitor,
						dlgX,
						dlgY,
						ro);
		s_dirty &= ~b;
	}

	// Let's draw the cursor.
	if (s_mode == MODE_RECEIVE_STRING) {
		if (((u8)(GetCPUSeconds() * 4)) & 1) {
			c = g_monitor.MapColor(1, 1, 0);
		} else {
			c = 0;
		}
		fw = TextGetFontWidth(g_font);
		x = fw * (g_userString.Length() - 1);
		x += dlgX + TEXT_X;
		y = TEXT_Y + dlgY;
		y += TextGetFontHeight(g_font);
		DrawLine(g_monitor,
				 c,
				 x,
				 y,
				 x + fw,
				 y);
	}
}
