/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	This is the implementation of the actual GBA device.

****************************************************************************/

#include "gba.h"

extern "C" {
	GBA g_gba;
	U8 * g_readMap[0x10];
	U8 * g_writeMap[0x10];
	U32 g_readBounds[0x10];
	U8 * g_readBPMap[0x10];
	U32 g_writeBounds[0x10];
	U32 g_memMask[0x10];
	U8 * g_writeBPMap[0x10];
	U8 * g_codeBPMap[0x10];
	U8 * g_userRBPMap[0x10];
	U8 * g_userWBPMap[0x10];
	U8 * g_userCBPMap[0x10];
	u32 g_readWaitStates[0x10];
	u32 g_writeWaitStates[0x10];

	U32 g_wordS[0x10];
	U32 g_wordN[0x10];
	U32 g_dwordS[0x10];
	U32 g_dwordN[0x10];

	U32 g_history[HISTORY_BUFFER_SIZE / 4];
	MType g_front;
	MType g_cursor = 0xffffffff;
	U32 g_pendingTimerInterrupts;
	U8 g_timerUpdateCode[1024];
};

u32 g_gbaTicks;

/////////////////////////////////////////////////////////////////////////////
//  Load in a ROM image.
void GBA::Load(const FString & filename) {
	Reset();
	m_systemMemory.LoadROM(filename);
	g_lastLoadedRom = filename;
}

/////////////////////////////////////////////////////////////////////////////
//  Reset the GBA.
void GBA::Reset(void) {
	g_gbaTicks = 0;
	m_systemMemory.Reset();
	m_processor.Reset();
}

/////////////////////////////////////////////////////////////////////////////
//  Get the symbol associated with this address.
void GBA::GetSymbol(FString & dest, u32 address) {
	char buf[256];

	sprintf(buf, "%08x", (unsigned)address);
	dest += buf;
}

/////////////////////////////////////////////////////////////////////////////
//  Make a GBA.
GBA::GBA(void) : m_processor(m_systemMemory) {
	m_video.SetParent(this);
	g_front = 0;
}

/////////////////////////////////////////////////////////////////////////////
//  Tick the GBA clock.
void GBA::Tick(void) {
	if (m_dma.Tick(0))
		m_processor.Tick();
}

/////////////////////////////////////////////////////////////////////////////
//  Tick the GBA backwards.
void GBA::TickBackward(void) {
	m_dma.TickBackward();
	m_processor.TickBackward();
}

/////////////////////////////////////////////////////////////////////////////
//  Convert the symbol to an address.
u32 GBA::GetAddress(const FString symbol) {
	return ConvertToHex(symbol);
}

/////////////////////////////////////////////////////////////////////////////
//  Shutdown in the AGD sense.
void GBA::ShutDown(void) {
	m_video.DeInitialize();
}
