/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	These contain certain globally-needed values.

****************************************************************************/

#ifndef GBAEMU_H
	#define GBAEMU_H

	/////////////////////////////////////////////////////////////////////////
	//  These are the exceptions that interrupt the GBA emulation.
	enum {
		EXC_ATOMIC				= 0x00000001,	// For single-cycle.
		EXC_RETIRED				= 0x00000002,	// Qualified by g_stepOut.
		EXC_TIMER				= 0x00000004,	// Timer interrupt generated.

		EXC_READ_BREAKPOINT		= 0x00000008,	// Data read breakpoint condition met.
		EXC_WRITE_BREAKPOINT	= 0x00000010,	// Data write breakpoint condition met.
		EXC_EXEC_BREAKPOINT		= 0x00000020,	// Code breakpoint condition met.
		EXC_USER_BREAKPOINT		= 0x00000040,	// User breakpoint condition met.

		EXC_INVALID_REG			= 0x00000080,	// User-controlled:  invalid register.
		EXC_BOUNDS_FAILURE		= 0x00000100,	// User-controlled:  memory read/write out of bounds.
		EXC_ABORT_AMBIGUOUS		= 0x00000200,	// User-controlled:  abort processing may be ambiguous.
		EXC_INVALID_OPCODE		= 0x00000400,	// User-controlled:  erroneous opcode (for some reason).

		EXC_SWI					= 0x00000800,	// CPU:  A software interrupt is thrown.
		EXC_UNDEFINED			= 0x00001000,	// CPU:  An undefined instruction is thrown.

		EXC_COPROCESSOR			= 0x00002000,	// Internal:  a coprocessor instruction is issued.
		EXC_STATE_CHANGE		= 0x00004000,	// Internal:  state may change (from bx).
		EXC_EXTERNAL_JUMP		= 0x00008000,	// Internal:  jump outside of cache line.
		EXC_CACHE_TERMINAL		= 0x00010000,	// Internal:  reached end of cache line.
		EXC_MODE_CHANGE			= 0x00020000,	// Internal:  mode may change.
		EXC_HISTORY_END			= 0x00040000,	// Internal:  end of history buffer reached.
		EXC_USER_MESSAGE		= 0x00080000,	// Internal:  user debug output found.

		// Flag can't exceed      0x00080000.  Bear it in mind!

		// Note:  if need be, we can lower this bit to make more space.
		EXC_CYCLE_BREAK			= 0x80000000,	// Process events (poll keys, video)
	};

	// These are the kind of conditions that throw us into the debugger.
	#define EXC_USER			(EXC_READ_BREAKPOINT |									\
								 EXC_WRITE_BREAKPOINT |									\
								 EXC_EXEC_BREAKPOINT |									\
								 EXC_ATOMIC			 |									\
								 EXC_RETIRED		 |									\
								 EXC_INVALID_REG	 |									\
								 EXC_BOUNDS_FAILURE	 |									\
								 EXC_ABORT_AMBIGUOUS |									\
								 EXC_INVALID_OPCODE	 |									\
								 EXC_USER_BREAKPOINT)
	// The kind of conditions that we normally want to monitor.  Note:  cycle_break,
	// which we do choose to monitor, behaves differently. . .
	#define EXC_NORMAL			(EXC_EXEC_BREAKPOINT |									\
								 EXC_USER_BREAKPOINT |									\
								 EXC_TIMER			 |									\
								 EXC_SWI			 |									\
								 EXC_UNDEFINED		 |									\
								 EXC_COPROCESSOR	 |									\
								 EXC_STATE_CHANGE	 |									\
								 EXC_EXTERNAL_JUMP   |									\
								 EXC_CACHE_TERMINAL	 |									\
								 EXC_MODE_CHANGE	 |									\
								 EXC_USER_MESSAGE	 |									\
								 EXC_HISTORY_END)
	
	// For now.
	#define EMU_WINDOW_WIDTH	(244)
	#define EMU_WINDOW_HEIGHT	(160 + TextGetFontHeight(g_font) + 2)
	// The video mode used when rendering fullscreen.
	#define EMU_WIDTH			320
	#define EMU_HEIGHT			240
	#define EMU_DEPTH			2

	#define STATUS_BAR_HEIGHT	(TextGetFontHeight(g_font))

	extern FontHandle g_font;

	// A little helper.
	inline u32 Ror(u32 val, u32 shift) { if (shift > 7) { val <<= 32 - shift; } else { val = (val >> shift) | (val << (32 - shift)); } return val; }

	/////////////////////////////////////////////////////////////////////////
	//  These are the modes of operation that the program might find itself
	// in.
	enum {
		MODE_DEBUGGER			= 0,
		MODE_APP,
		MODE_QUERY_USER
	};

	extern void SetMode(u8 mode);

#endif
