/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	This is the implementation of GBA sound.

****************************************************************************/

#include "gba.h"

#define MAX_SAMPLES_PER_FRAME			2048

static U16 s_soundBuffer[4][MAX_SAMPLES_PER_FRAME];
static MType s_soundCursor;
static u32 s_lastTicks;
static u32 s_lastCursor;

Sticky < Bool > g_soundEnable("Sound Enable", TRUE);

/////////////////////////////////////////////////////////////////////////////
//  Update all sounds.
void GBASoundUpdate(void) {
	if (*g_soundEnable) {
		MType m;
	
		for (m=0; m<4; ++m) {
			g_soundManager.Insert(m, s_soundCursor, &s_soundBuffer[m][0]);
		}
		s_soundCursor = 0;
		s_lastCursor = 0;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Add the indicated samples.
void GBASoundAddSamples(u16 v0, u16 v1, u16 v2, u16 v3) {
	if (*g_soundEnable) {
		u32 ticks = g_gba.GetTicks();
		u32 cursor = s_lastCursor + (ticks - s_lastTicks) * 86;
		MType c = (cursor >> 16) - (s_lastCursor >> 16);
	
		while (c--) {
			s_soundBuffer[0][s_soundCursor] = v0;
			s_soundBuffer[1][s_soundCursor] = v1;
			s_soundBuffer[2][s_soundCursor] = v2;
			s_soundBuffer[3][s_soundCursor] = v3;
			++s_soundCursor;
		}
		s_lastTicks = ticks;
		s_lastCursor = cursor;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Turn sound on.
void GBASoundOn(void) {
	g_soundManager.Initialize();
}

/////////////////////////////////////////////////////////////////////////////
//  Turn sound off.
void GBASoundOff(void) {
	g_soundManager.Shutdown();
}
