/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	The implementation of much of the history stuff.

****************************************************************************/

#include "gba.h"

u8 g_orientation;

/////////////////////////////////////////////////////////////////////////////
//  Initialize the history system.
void HistoryInitialize(void) {
	MType m;

	for (m=0; m<HISTORY_BUFFER_SIZE / 4; ++m) {
		g_history[m] = HISTORY_TERMINAL;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Is there any history prior to my current state?
Bool HistoryExists(void) {
	return g_cursor != g_front;
}

/////////////////////////////////////////////////////////////////////////////
//  Get the next preceding address-value pair.  Return FALSE if we've hit
// our limit.
Bool HistoryGetPrior(u32 & a, u32 & v) {
	if (g_cursor == 0xffffffff) {
		g_cursor = g_front;
	}
	g_cursor = (g_cursor - 2) & (HISTORY_BUFFER_SIZE / 4 - 1);
	if (g_cursor == g_front) {
		return FALSE;
	} else {
		a = g_history[g_cursor];
		v = g_history[g_cursor + 1];
	}
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
//  Get the next succeeding address-value pair.  Return FALSE if we've hit
// our limit.
Bool HistoryGetNext(u32 & a, u32 & v) {
	if (g_cursor == 0xffffffff) {
		return FALSE;
	}
	a = g_history[g_cursor];
	v = g_history[g_cursor + 1];
	g_cursor = (g_cursor + 2) & (HISTORY_BUFFER_SIZE / 4 - 1);
	if (g_cursor == g_front) {
		g_cursor = 0xffffffff;
	}
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
//  Specify the direction of time flow.
void HistorySetOrientation(u8 orientation) {
	g_orientation = orientation;
	g_gba.GetVideo().SetOrientation(orientation);
}

/////////////////////////////////////////////////////////////////////////////
//  Save the indicated value into the history buffer.
void HistorySave(U32 * location) {
	g_history[g_front] = (u32)location;
	g_history[g_front + 1] = *location;
	g_front = g_front + 2;
	if (!(g_front &= HISTORY_BUFFER_SIZE / 4 - 1)) {
		HistoryFlush();
	}
}
