/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	The implementation of memory.

****************************************************************************/

#include "gba.h"

// HEREHEREHERE we also have to implement "prefetch" support.
#define IO_RAM_WORD_S_STATES				1		// HEREHEREHERE variable.
#define EXTERNAL_RAM_WORD_S_STATES			3
#define INTERNAL_RAM_WORD_S_STATES			1
#define ROM_WORD_S_STATES					3		// HEREHEREHERE actually, selectable.
#define PALETTE_RAM_WORD_S_STATES			2		// Confirmed.
#define VRAM_WORD_S_STATES					2		// Confirmed.
#define OAM_RAM_WORD_S_STATES				1		// HEREHEREHERE experiment.
#define CART_RAM_WORD_S_STATES				6		// HEREHEREHERE untested.
#define EEPROM_WORD_S_STATES				6		// HEREHEREHERE untested.
#define MISC_WORD_S_STATES					4		// HEREHEREHERE untested.
#define BIOS_ROM_WORD_S_STATES				6		// HEREHEREHERE untested.

#define IO_RAM_WORD_N_STATES				1		// HEREHEREHERE variable.
#define EXTERNAL_RAM_WORD_N_STATES			3
#define INTERNAL_RAM_WORD_N_STATES			1
#define ROM_WORD_N_STATES					5		// HEREHEREHERE actually, selectable.
#define PALETTE_RAM_WORD_N_STATES			2		// Confirmed.
#define VRAM_WORD_N_STATES					2		// Confirmed.
#define OAM_RAM_WORD_N_STATES				1		// HEREHEREHERE experiment.
#define CART_RAM_WORD_N_STATES				6		// HEREHEREHERE untested.
#define EEPROM_WORD_N_STATES				6		// HEREHEREHERE untested.
#define MISC_WORD_N_STATES					4		// HEREHEREHERE untested.
#define BIOS_ROM_WORD_N_STATES				6		// HEREHEREHERE untested.

#define IO_RAM_DWORD_S_STATES				1		// HEREHEREHERE variable.
#define EXTERNAL_RAM_DWORD_S_STATES			6
#define INTERNAL_RAM_DWORD_S_STATES			1
#define ROM_DWORD_S_STATES					6		// HEREHEREHERE actually, selectable.
#define PALETTE_RAM_DWORD_S_STATES			2		// Confirmed.
#define VRAM_DWORD_S_STATES					2		// Confirmed.
#define OAM_RAM_DWORD_S_STATES				1		// HEREHEREHERE experiment.
#define CART_RAM_DWORD_S_STATES				6		// HEREHEREHERE untested.
#define EEPROM_DWORD_S_STATES				6		// HEREHEREHERE untested.
#define MISC_DWORD_S_STATES					4		// HEREHEREHERE untested.
#define BIOS_ROM_DWORD_S_STATES				6		// HEREHEREHERE untested.

#define IO_RAM_DWORD_N_STATES				1		// HEREHEREHERE variable.
#define EXTERNAL_RAM_DWORD_N_STATES			6
#define INTERNAL_RAM_DWORD_N_STATES			1
#define ROM_DWORD_N_STATES					8		// HEREHEREHERE actually, selectable.
#define PALETTE_RAM_DWORD_N_STATES			2		// Confirmed.
#define VRAM_DWORD_N_STATES					2		// Confirmed.
#define OAM_RAM_DWORD_N_STATES				1		// HEREHEREHERE experiment.
#define CART_RAM_DWORD_N_STATES				6		// HEREHEREHERE untested.
#define EEPROM_DWORD_N_STATES				6		// HEREHEREHERE untested.
#define MISC_DWORD_N_STATES					4		// HEREHEREHERE untested.
#define BIOS_ROM_DWORD_N_STATES				6		// HEREHEREHERE untested.

/////////////////////////////////////////////////////////////////////////////
//  The memory controller for the GBA.
MemoryDispatcher::MemoryDispatcher(void) {
	MType m;
	U8 * bp;
	static U8 ** s_maps[] = {
		g_readBPMap,
		g_writeBPMap,
		g_codeBPMap,
		g_userRBPMap,
		g_userWBPMap,
		g_userCBPMap,
	};
	U8 * srcMaps[] = {
		m_readBreakpoints,
		m_writeBreakpoints,
		m_codeBreakpoints,
		m_userRead,
		m_userWrite,
		m_userCode,
	};

	for (m=0; m<0x10; ++m) {
		g_readMap[m] = g_writeMap[m] = NULL;
		g_readBounds[m] = g_writeBounds[m] = 0;
		g_readBPMap[m] = g_writeBPMap[m] = g_codeBPMap[m] =
		g_userRBPMap[m] = g_userWBPMap[m] = g_userCBPMap[m] = NULL;
	}
	g_readBounds[0x0] = MEM_BIOS_SIZE;
	g_readBounds[0x2] = g_writeBounds[0x2] = MEM_EXTERNAL_RAM_SIZE;
	g_readBounds[0x3] = g_writeBounds[0x3] = MEM_INTERNAL_RAM_SIZE;
	g_readBounds[0x4] = g_writeBounds[0x4] = MEM_MMAP_RAM_SIZE;
	g_readBounds[0x5] = g_writeBounds[0x5] = MEM_PALETTE_RAM_SIZE;
	g_readBounds[0x6] = g_writeBounds[0x6] = MEM_VRAM_RAM_SIZE;
	g_readBounds[0x7] = g_writeBounds[0x7] = MEM_OAM_SIZE;
	g_readBounds[0x8] = g_readBounds[0xa] = g_readBounds[0xc] = MEM_CODE_ROM_SIZE;
	g_readBounds[0x9] = g_readBounds[0xb] = g_readBounds[0xd] =
	g_writeBounds[0x9] = g_writeBounds[0xb] = g_writeBounds[0xd] = MEM_EEPROM_SIZE;
	g_readBounds[0xe] = g_writeBounds[0xe] = MEM_CART_RAM_SIZE;
// HEREHEREHERE is above mirrored to F?

	for (m=0; m<6; ++m) {
		U8 ** dest = s_maps[m];

		bp = srcMaps[m];
		dest[0x0] = bp;
		bp += MEM_BIOS_SIZE / 32;
		dest[0x2] = bp;
		bp += MEM_EXTERNAL_RAM_SIZE / 32;
		dest[0x3] = bp;
		bp += MEM_INTERNAL_RAM_SIZE / 32;
		dest[0x4] = bp;
		bp += MEM_MMAP_RAM_SIZE / 32;
		dest[0x5] = bp;
		bp += MEM_PALETTE_RAM_SIZE / 32;
		dest[0x6] = bp;
		bp += MEM_VRAM_RAM_SIZE / 32;
		dest[0x7] = bp;
		bp += MEM_OAM_SIZE / 32;
		dest[0x8] =
		dest[0xa] =
		dest[0xc] = bp;
		bp += MEM_CODE_ROM_SIZE / 32;
		dest[0x9] =
		dest[0xb] =
		dest[0xd] = bp;
		bp += MEM_EEPROM_SIZE / 32;
		dest[0xe] = bp;
	}

	g_readMap[0x0] = (U8 *)m_bios.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0x2] = (U8 *)m_externalRam.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0x3] = (U8 *)m_internalRam.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0x4] = (U8 *)m_mmapRegs.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0x5] = (U8 *)m_palettes.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0x6] = (U8 *)m_vram.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0x7] = (U8 *)m_oam.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0x8] = 
	g_readMap[0xa] =
	g_readMap[0xc] = (U8 *)m_code.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0x9] =
	g_readMap[0xb] =
	g_readMap[0xd] = (U8 *)m_eeprom.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_readMap[0xe] = (U8 *)m_cartRam.GetVoidPointerOnlyIfYouReallyNeedIt();

	g_writeMap[0x2] = (U8 *)m_externalRam.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_writeMap[0x3] = (U8 *)m_internalRam.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_writeMap[0x4] = (U8 *)m_mmapRegs.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_writeMap[0x5] = (U8 *)m_palettes.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_writeMap[0x6] = (U8 *)m_vram.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_writeMap[0x7] = (U8 *)m_oam.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_writeMap[0x9] =
	g_writeMap[0xb] =
	g_writeMap[0xd] = (U8 *)m_eeprom.GetVoidPointerOnlyIfYouReallyNeedIt();
	g_writeMap[0xe] = (U8 *)m_cartRam.GetVoidPointerOnlyIfYouReallyNeedIt();

	// Set up the wait-states for the different memories.  HEREHEREHERE we
	// need to get an accurate measurement of these.
	g_wordS[0x0] = BIOS_ROM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x1] = MISC_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x2] = EXTERNAL_RAM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x3] = INTERNAL_RAM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x4] = IO_RAM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x5] = PALETTE_RAM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x6] = VRAM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x7] = OAM_RAM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x8] = ROM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0x9] = EEPROM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0xa] = ROM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0xb] = EEPROM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0xc] = ROM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0xd] = EEPROM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0xe] = CART_RAM_WORD_S_STATES << CYCLE_SHIFT;
	g_wordS[0xf] = MISC_WORD_S_STATES << CYCLE_SHIFT;

	g_wordN[0x0] = BIOS_ROM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x1] = MISC_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x2] = EXTERNAL_RAM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x3] = INTERNAL_RAM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x4] = IO_RAM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x5] = PALETTE_RAM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x6] = VRAM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x7] = OAM_RAM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x8] = ROM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0x9] = EEPROM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0xa] = ROM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0xb] = EEPROM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0xc] = ROM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0xd] = EEPROM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0xe] = CART_RAM_WORD_N_STATES << CYCLE_SHIFT;
	g_wordN[0xf] = MISC_WORD_N_STATES << CYCLE_SHIFT;

	g_dwordS[0x0] = BIOS_ROM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x1] = MISC_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x2] = EXTERNAL_RAM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x3] = INTERNAL_RAM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x4] = IO_RAM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x5] = PALETTE_RAM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x6] = VRAM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x7] = OAM_RAM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x8] = ROM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0x9] = EEPROM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0xa] = ROM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0xb] = EEPROM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0xc] = ROM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0xd] = EEPROM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0xe] = CART_RAM_DWORD_S_STATES << CYCLE_SHIFT;
	g_dwordS[0xf] = MISC_DWORD_S_STATES << CYCLE_SHIFT;

	g_dwordN[0x0] = BIOS_ROM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x1] = MISC_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x2] = EXTERNAL_RAM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x3] = INTERNAL_RAM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x4] = IO_RAM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x5] = PALETTE_RAM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x6] = VRAM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x7] = OAM_RAM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x8] = ROM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0x9] = EEPROM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0xa] = ROM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0xb] = EEPROM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0xc] = ROM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0xd] = EEPROM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0xe] = CART_RAM_DWORD_N_STATES << CYCLE_SHIFT;
	g_dwordN[0xf] = MISC_DWORD_N_STATES << CYCLE_SHIFT;

	for (m=0; m<0x10; ++m) {
		if (g_readBounds[m]) {
			g_memMask[m] = g_readBounds[m] - 1;
		} else {
			g_memMask[m] = 0xffffff;
		}
	}

	// Some memory-mapped addresses have some initialized values.
	SetW(0x4000130, 0x3ff);
	SetW(0xd000000, 1);

	// Now we're ready to set the internal breakpoints.
	BreakpointSetAllInternal();

	// Finally, load in the GBA bios.
	m_bios.Load("gba.rom");
}

/////////////////////////////////////////////////////////////////////////////
//  Reset the memory - basically, clear out all the garbage.
void MemoryDispatcher::Reset(void) {
	memset(g_readMap[2], 0, MEM_EXTERNAL_RAM_SIZE);
	memset(g_readMap[3], 0, MEM_INTERNAL_RAM_SIZE);
	memset(g_readMap[4], 0, MEM_MMAP_RAM_SIZE);
	memset(g_readMap[5], 0, MEM_PALETTE_RAM_SIZE);
	memset(g_readMap[6], 0, MEM_VRAM_RAM_SIZE);
	memset(g_readMap[7], 0, MEM_OAM_SIZE);
	memset(g_readMap[9], 0, MEM_EEPROM_SIZE);
	memset(g_readMap[0xe], 0, MEM_CART_RAM_SIZE);

	// Some memory-mapped addresses have some initialized values.
	SetW(0x4000130, 0x3ff);
	SetW(0xd000000, 1);
}

/////////////////////////////////////////////////////////////////////////////
//  Load a ROM.
void MemoryDispatcher::LoadROM(const FString & filename) {
	m_code.Load(filename);
}

/////////////////////////////////////////////////////////////////////////////
//  The RegisterSet constructor initializes all of the bank pointers.
RegisterSet::RegisterSet(void) {
	MType m, n;
	U32 * d;
	static U32 s_dummy;

	d = m_regs;

	// Initialize them to be all the same.
	for (m=0; m<20; ++m) {
		for (n=0; n<7; ++n) {
			m_refs[m][n] = d;
		}
		++d;
	}

	// Let's give FIQ its own set of registers.
	for (m=8; m<15; ++m) {
		m_refs[m][1] = d++;
	}
	m_refs[SPSR][1] = d++;
	m_refs[HR1][1] = d++;
	m_refs[HR2][1] = d++;

	// Now let's do the same for the remaining modes.
	for (n=2; n<7; ++n) {
		for (m=13; m<15; ++m) {
			m_refs[m][n] = d++;
		}
		m_refs[SPSR][n] = d++;
		m_refs[HR1][n] = d++;
		m_refs[HR2][n] = d++;
	}

	CHECK(d - m_regs == REGISTER_SIZE);

	// Okay!  Everybody is all banked out now.  :)
}

/////////////////////////////////////////////////////////////////////////////
//  Set the indicated register to the indicated value, and save the old value
// in the history buffer.
void RegisterSet::SetHistory(u8 index, u8 mode, u32 value) {
	U32 * upt = GetAddress(index, mode);

	HistorySave(upt);
	*upt = value;
}

/////////////////////////////////////////////////////////////////////////////
//  Blank out all of the registers.
void RegisterSet::Reset(void) {
	memset(m_regs, 0, REGISTER_SIZE * sizeof(U32));
}
