/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	This class is a very simple class for holding memory.  We add a few odds
and ends for printing out messages, etc.

****************************************************************************/

#ifndef MEMORY_H
	#define MEMORY_H

	#define BAD_ACCESS					0xffffffffUL

	#define CPSR				16
	#define SPSR				17
	#define HR1					18			// Hidden register (for internal use)
	#define HR2					19			// Hidden register (for internal use)
	#define REGISTER_SIZE		55

	#define MEM_BIOS_SIZE					0x4000
	#define MEM_EXTERNAL_RAM_SIZE			0x40000
	#define MEM_INTERNAL_RAM_SIZE			0x8000
	#define MEM_MMAP_RAM_SIZE				0x400
	#define MEM_PALETTE_RAM_SIZE			0x400
	#define MEM_VRAM_RAM_SIZE				0x20000
	#define MEM_OAM_SIZE					0x4000
	#define MEM_CODE_ROM_SIZE				0x1000000
	#define MEM_CART_RAM_SIZE				0x10000
	#define MEM_EEPROM_SIZE					0x100
	#define MEM_TOTAL_SIZE					(MEM_BIOS_SIZE +					\
											 MEM_EXTERNAL_RAM_SIZE +			\
											 MEM_INTERNAL_RAM_SIZE +			\
											 MEM_MMAP_RAM_SIZE +				\
											 MEM_PALETTE_RAM_SIZE +				\
											 MEM_VRAM_RAM_SIZE +				\
											 MEM_OAM_SIZE +						\
											 MEM_CODE_ROM_SIZE +				\
											 MEM_CART_RAM_SIZE +				\
											 MEM_EEPROM_SIZE)

	/////////////////////////////////////////////////////////////////////////
	//	Simplifying the heck out of memory.
	template < MType t_size >
	class Memory : public Vec < U8, t_size > {
		public:
			void Load(const FString & filename);
			void * GetVoidPointerOnlyIfYouReallyNeedIt(void);
		protected:
		private:
	};

	/////////////////////////////////////////////////////////////////////////
	//	This class represents the memory dispatcher for the
	// GBA.
	class MemoryDispatcher {
		public:
			MemoryDispatcher(void);

			void LoadROM(const FString & filename);
			void Reset(void);

			u8 GetB(u32 address);
			u16 GetW(u32 address);
			u32 GetD(u32 address);
			void SetB(u32 address, u8 value);
			void SetW(u32 address, u16 value);
			void SetD(u32 address, u32 value);
			void ForceB(u32 address, u8 value);
			void ForceW(u32 address, u8 value);
			void ForceD(u32 address, u8 value);
		protected:
		private:
			Memory < MEM_BIOS_SIZE			> m_bios;
			Memory < MEM_EXTERNAL_RAM_SIZE	> m_externalRam;
			Memory < MEM_INTERNAL_RAM_SIZE	> m_internalRam;
			Memory < MEM_MMAP_RAM_SIZE		> m_mmapRegs;
			Memory < MEM_PALETTE_RAM_SIZE	> m_palettes;
			Memory < MEM_VRAM_RAM_SIZE		> m_vram;
			Memory < MEM_OAM_SIZE			> m_oam;
			Memory < MEM_CODE_ROM_SIZE		> m_code;
			Memory < MEM_CART_RAM_SIZE		> m_cartRam;
			Memory < MEM_EEPROM_SIZE		> m_eeprom;
			U8 m_readBreakpoints[MEM_TOTAL_SIZE / 32];
			U8 m_writeBreakpoints[MEM_TOTAL_SIZE / 32];
			U8 m_codeBreakpoints[MEM_TOTAL_SIZE / 32];
			U8 m_userRead[MEM_TOTAL_SIZE / 32];
			U8 m_userWrite[MEM_TOTAL_SIZE / 32];
			U8 m_userCode[MEM_TOTAL_SIZE / 32];
	};

	/////////////////////////////////////////////////////////////////////////
	//	Okay, here we go.  This new implementation of the register set will
	// account for the banking supported on ARM.
	class RegisterSet {
		public:
			RegisterSet(void);

			void Reset(void);
			u32 Get(u8 index, u8 mode) const;
			void Set(u8 index, u8 mode, u32 value);
			void SetHistory(u8 index, u8 mode, u32 value);
			U32 * GetAddress(u8 index, u8 mode);
			const U32 * GetAddress(u8 index, u8 mode) const;
			u8 GetMode(u8 mode) const;	//  Convert from ARM to me.
		protected:
		private:
			U32 * m_refs[20][8];		//  Actually less, but is power of 2.
			U32 m_regs[REGISTER_SIZE];
	};

	#include "memory.ipp"

#endif

