/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	Inlines, etc. for memory.

****************************************************************************/

extern "C" {
	extern U8 * g_readMap[];
	extern U8 * g_writeMap[];
	extern U32 g_readBounds[];
	extern U32 g_writeBounds[];
	extern U32 g_memMask[];			// HEREHEREHERE have to put bounds
};

/////////////////////////////////////////////////////////////////////////////
//  For asm interfacing.
template < MType t_size >
void * Memory < t_size >::GetVoidPointerOnlyIfYouReallyNeedIt(void) {
	return (void *)m_elements;
}

/////////////////////////////////////////////////////////////////////////////
//  Load a memory image.
template < MType t_size >
void Memory < t_size >::Load(const FString & filename) {
	Fill(DirLoadFile(filename));
}

/////////////////////////////////////////////////////////////////////////////
//  Fetch a byte.
inline u8 MemoryDispatcher::GetB(u32 address) {
	MType ind = (address >> 24) & 0xff;

	if (ind > 0xf) {
		return (U8)BAD_ACCESS;
	}
	address &= g_memMask[ind];

	if (g_readMap[ind]) {
		if (address >= g_readBounds[ind]) {
			return (U8)BAD_ACCESS;
		} else {
			return *((U8 *)(&g_readMap[ind][address]));
		}
	} else {
		return (U8)BAD_ACCESS;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Get a word from the indicated location.
inline u16 MemoryDispatcher::GetW(u32 address) {
	MType ind = (address >> 24) & 0xff;

	if (ind > 0xf) {
		return (U16)BAD_ACCESS;
	}
	address &= g_memMask[ind];

	if (g_readMap[ind]) {
		if (address >= g_readBounds[ind] - 1) {
			return (U16)BAD_ACCESS;
		} else {
			return *((U16 *)(&g_readMap[ind][address]));
		}
	} else {
		return (U16)BAD_ACCESS;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Get a word from the indicated location.
inline u32 MemoryDispatcher::GetD(u32 address) {
	MType ind = (address >> 24) & 0xff;

	if (ind > 0xf) {
		return BAD_ACCESS;
	}
	address &= g_memMask[ind];

	if (g_readMap[ind]) {
		if (address >= g_readBounds[ind] - 3) {
			return BAD_ACCESS;
		} else {
			return *((U32 *)(&g_readMap[ind][address]));
		}
	} else {
		return BAD_ACCESS;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Set the byte in the indicated location.
inline void MemoryDispatcher::SetB(u32 address, u8 value) {
	MType ind = (address >> 24) & 0xff;

	// HEREHEREHERE need to throw error condition?
	if (ind > 0xf) {
		return;
	}
	address &= g_memMask[ind];

	if (g_writeMap[ind]) {
		if (address >= g_writeBounds[ind]) {
			return;
		} else {
			*((U8 *)(&g_writeMap[ind][address])) = value;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Set the word in the indicated location.
inline void MemoryDispatcher::SetW(u32 address, u16 value) {
	MType ind = (address >> 24) & 0xff;

	// HEREHEREHERE need to throw error condition?
	if (ind > 0xf) {
		return;
	}
	address &= g_memMask[ind];

	if (g_writeMap[ind]) {
		if (address >= g_writeBounds[ind] - 1) {
			return;
		} else {
			*((U16 *)(&g_writeMap[ind][address])) = value;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Set the dword in the indicated location.
inline void MemoryDispatcher::SetD(u32 address, u32 value) {
	MType ind = (address >> 24) & 0xff;

	// HEREHEREHERE need to throw error condition?
	if (ind > 0xf) {
		return;
	}
	address &= g_memMask[ind];

	if (g_writeMap[ind]) {
		if (address >= g_writeBounds[ind] - 3) {
			return;
		} else {
			*((U32 *)(&g_writeMap[ind][address])) = value;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Force the byte in the indicated location.
inline void MemoryDispatcher::ForceB(u32 address, u8 value) {
	MType ind = (address >> 24) & 0xff;

	// HEREHEREHERE need to throw error condition?
	if (ind > 0xf) {
		return;
	}
	address &= g_memMask[ind];

	if (g_readMap[ind]) {
		if (address >= g_readBounds[ind]) {
			return;
		} else {
			*((U8 *)(&g_readMap[ind][address])) = value;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Force the word in the indicated location.
inline void MemoryDispatcher::ForceW(u32 address, u16 value) {
	MType ind = (address >> 24) & 0xff;

	// HEREHEREHERE need to throw error condition?
	if (ind > 0xf) {
		return;
	}
	address &= g_memMask[ind];

	if (g_readMap[ind]) {
		if (address >= g_readBounds[ind] - 1) {
			return;
		} else {
			*((U16 *)(&g_readMap[ind][address])) = value;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Force the dword in the indicated location.
inline void MemoryDispatcher::ForceD(u32 address, u32 value) {
	MType ind = (address >> 24) & 0xff;

	// HEREHEREHERE need to throw error condition?
	if (ind > 0xf) {
		return;
	}
	address &= g_memMask[ind];

	if (g_readMap[ind]) {
		if (address >= g_readBounds[ind] - 3) {
			return;
		} else {
			*((U32 *)(&g_readMap[ind][address])) = value;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Convert the ARM mode to my mode.  HEREHEREHERE this makes no bones about
// bad modes.
inline u8 RegisterSet::GetMode(u8 mode) const {
	mode &= 0xf;

	switch(mode) {
		default:
			mode = 0;
		case 0:
		case 1:
		case 2:
		case 3:
			break;
		case 7:
			mode = 4;
			break;
		case 11:
			mode = 5;
			break;
		case 15:
			mode = 6;
			break;
	}
	return mode;
}

/////////////////////////////////////////////////////////////////////////////
//  Dereference the indicated register.
inline u32 RegisterSet::Get(u8 index, u8 mode) const {
	return *GetAddress(index, mode);
}

/////////////////////////////////////////////////////////////////////////////
//  Set the indicated register to the indicated value.
inline void RegisterSet::Set(u8 index, u8 mode, u32 value) {
	*GetAddress(index, mode) = value;
}

/////////////////////////////////////////////////////////////////////////////
//  Get the address of the indicated register.
inline U32 * RegisterSet::GetAddress(u8 index, u8 mode) {
	return m_refs[index][GetMode(mode)];
}

/////////////////////////////////////////////////////////////////////////////
//  Get the address of the indicated register.
inline const U32 * RegisterSet::GetAddress(u8 index, u8 mode) const {
	return m_refs[index][GetMode(mode)];
}
