/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	This handles all of the "sticky" options.

****************************************************************************/

#include "gba.h"

inline void ConvertTo(const char * s, int & i) {
	i = atoi(s);
}

inline void ConvertFrom(char * s, int i) {
	sprintf(s, "%d", i);
}

inline void ConvertTo(const char * s, unsigned & i) {
	i = atoi(s);
}

inline void ConvertTo(const char * s, U16 & i) {
	i = atoi(s);
}

inline void ConvertTo(const char * s, FString & f) {
	f = s;
}

inline void ConvertFrom(char * s, const FString & f) {
	CopyToChar(f, s, 256);
}

/////////////////////////////////////////////////////////////////////////////
//  Specify a sticky option.
template < class C >
Sticky < C >::Sticky(const char * name, C def) : m_name(name) {
	FILE * f;
	char buf[1024];
	char * toks[2];

	m_curValue = def;
	if (f = fopen("agd.cfg", "r")) {
		while (!feof(f)) {
			fgets(buf, 1024, f);
			toks[0] = strtok(buf, "\t");
			toks[1] = strtok(NULL, "\t\r\n");
			if (toks[0] && toks[1] && (!strcmp(toks[0], m_name))) {
				ConvertTo(toks[1], m_curValue);
				break;
			}
		}
		fclose(f);
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Delete the sticky option.
template < class C >
Sticky < C >::~Sticky(void) {
	FILE * f, * g;
	char buf[1024];
	char * toks[2];
	char filename[MAX_PATH];
	Bool needToWrite = TRUE;

	if (GetTempFileName("./",
						"agd",
						0,
						filename) == 0) {
		return;
	}
	if (!(g = fopen(filename, "w"))) {
		return;
	}
	if (f = fopen("agd.cfg", "r")) {
		while (!feof(f)) {
			if (!fgets(buf, 1024, f))
				break;
			toks[0] = strtok(buf, "\t");
			toks[1] = strtok(NULL, "\t\r\n");
			if (toks[0] && toks[1] && (!strcmp(toks[0], m_name))) {
				needToWrite = FALSE;
				ConvertFrom(buf, m_curValue);
				fprintf(g, "%s\t%s\n", m_name, buf);
			} else {
				fprintf(g, "%s\t%s\n", toks[0], toks[1]);
			}
		}
		fclose(f);
	}
	if (needToWrite) {
		ConvertFrom(buf, m_curValue);
		fprintf(g, "%s\t%s\n", m_name, buf);
	}
	fclose(g);
	DeleteFile("agd.cfg");
	MoveFile(filename, "agd.cfg");
}

/////////////////////////////////////////////////////////////////////////////
//  r-value.
template < class C >
inline const C & Sticky < C >::operator *(void) const {
	return m_curValue;
}

/////////////////////////////////////////////////////////////////////////////
//  l-value.
template < class C >
inline C & Sticky < C >::operator *(void) {
	return m_curValue;
}
