/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	This will emulate the video processor that the GBA uses.

****************************************************************************/

#ifndef VIDEO_H
	#define VIDEO_H

	class GBA;

	/////////////////////////////////////////////////////////////////////////
	//	This handles the logic for the video processor.  For now, I will not
	// be doing direct memory writes, instead I will blit scanlines as they
	// are rendered.
	class VideoProcessor {
		public:
			enum {
				TARGET_WINDOW,
				TARGET_FULLSCREEN,
				TARGET_FAST_WINDOW
			};
			enum {
				E_HBLANK,
				E_HTOTAL,
			};
			enum {
				VIDEO_IN_COINCIDENCE	= 0x04,
				VIDEO_IN_HBLANK			= 0x02,
				VIDEO_IN_VBLANK			= 0x01,
				VIDEO_IRQ_VCOUNT		= 0x20,
				VIDEO_IRQ_HBLANK		= 0x10,
				VIDEO_IRQ_VBLANK		= 0x08,
			};
			enum {
				V_RENDERING,
				V_HBLANK,
				V_VBLANK
			};
			VideoProcessor(void);
			void SetParent(GBA * parent);
			void Tick(void);
			void TickBackwards(void);
			Bool IsOn(void);
			void Redraw(void);
			void RenderScanline(const u8 v);
			u8 GetMode(void);
			void Initialize(void);
			u8 GetPeriod(void);

			void DeInitialize(void);

			//  When we switch modes, ALL graphical elements that depended
			// on the view configuration need to be reconfigured.
			void SetTarget(u8 target);
			void SetOrientation(u8 orientation);
			class GBSprite {
				public:
					U16 ySHi;
					U16 xSLo;
					U16 prioTile;
					S16 scaleRot;

					Bool operator ==(const GBSprite & s) const { return (ySHi == s.ySHi) && (xSLo == s.xSLo) && (prioTile == s.prioTile); }
					Bool operator !=(const GBSprite & s) const { return !(*this == s); }
			};
		protected:
		private:
			typedef U16 GBAColor;

			typedef void (*Renderer)(VideoProcessor * vp, const u8 v);

			enum {
				LCDC		= 0,
				STAT		= 2,
				LCY			= 3,
				BG0			= 4,
				BG1			= 5,
				BG2			= 6,
				BG3			= 7,
				SCX0		= 8,
				SCY0		= 9,
				BLDMOD		= 0x28,
				COLEV		= 0x29,
				COLY		= 0x2a,
			};

			// Stubs.
			void BuildSpriteIndirectionTables(void);
			void UpdateBGLayers(const u8 v);
			void UpdateSprite(MType v);
			void UpdateOAMLayers(const u8 v);
			void RenderItems(const u8 y);
			void AccumulateResults(const u8 v);

			static GBSprite * s_sprites[128];
			static Renderer s_renderers[8];
			GBA * m_parent;
			U16 * m_vidreg;
			GBAColor * m_pal;
			U8 * m_vram;

			GBSprite * m_sprites;
			GBAColor * m_spritePal;
			U8 * m_spriteData;

			Color m_debugMap[0x8000];
			Color m_appMap[0x8000];
			GBAColor m_alphaTable[0x8000 * 0xf];
			color m_debugMask;
			color m_appMask;
			u8 m_scanline;

			u8 m_nextEvent;
			u8 m_target;
			u8 m_orientation;
	};

	// These can be used to globally enable/disable BG layers or OAM.
	enum {
		ENABLE_OAM			= ((u16)1 << 12),
		ENABLE_BG3			= ((u16)1 << 11),
		ENABLE_BG2			= ((u16)1 << 10),
		ENABLE_BG1			= ((u16)1 << 9),
		ENABLE_BG0			= ((u16)1 << 8),
	};

	extern u16 g_ticksTilNextVideo;
	extern u16 g_globalEnable;
	extern Sticky < Bool > g_runAppModeSwitch;
	extern Sticky < Bool > g_runAppWindow;
	extern Sticky < u8 > g_grid;
	extern Sticky < U16 > g_userVideoWidth;
	extern Sticky < U16 > g_userVideoHeight;

	extern void VideoMakeScreenShot(void);

	#include "video.ipp"

#endif
