/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	The implementation of our first window.

****************************************************************************/

#ifndef WINDOW1_H
	#define WINDOW1_H

	/////////////////////////////////////////////////////////////////////////
	//	A FileTextPanel displays the contents of a file.
	class FileTextPanel : public IListTextPanel {
		public:
			void Open(const FString & filename);
			virtual void Redraw(void);
		protected:
		private:
	};

	/////////////////////////////////////////////////////////////////////////
	//	Browsers need special behavior for clicking.
	class BrowserPanel : public IListTextPanel {
		public:
			BrowserPanel(void);
			virtual void Click(u16 x, u16 y);
		protected:
		private:
			void FindAllFiles(const FString & s);
	};

	/////////////////////////////////////////////////////////////////////////
	//	We need to make a separate panel for the disassembler.
	class DisassemblyPanel : public TextPanel {
		public:
			virtual void Click(u16 x, u16 y);

			virtual void GenerateContent(void);
			virtual void SetDims(u16 x, u16 y, u16 w, u16 h);

			virtual void CursorUp(void);
			virtual void CursorDown(void);
			virtual void PageUp(void);
			virtual void PageDown(void);

			virtual void Goto(u32 line);
			virtual void Redraw(void);

			u32 GetLine(void) const;
		protected:
		private:
			u32 m_address;
			u32 m_bottom;
	};

	/////////////////////////////////////////////////////////////////////////
	//	This panel will enable the user to configure the debugger/emulator
	// options.
	class OptionsPanel : public Panel {
		public:
			OptionsPanel(void);
			virtual void SetDims(u16 x, u16 y, u16 w, u16 h);
			virtual void ResizeGUIElement(GUIElement * g);
			virtual void Redraw(void);
			virtual void CursorUp(void);
			virtual void CursorDown(void);
			virtual void PageUp(void);
			virtual void PageDown(void);
			virtual void GenerateContent(void);
		protected:
		private:
			#if !defined(RELEASE_BUILD)
				class Language : public GUIRadioCheckBox {
					public:
						Language(void);
						virtual u8 QuerySelect(void);
						virtual void Select(u8 item);
						virtual void Deselect(u8 item);
					protected:
					private:
				};
			#endif
			class LineCount : public GUIRadioCheckBox {
				public:
					LineCount(void);
					virtual u8 QuerySelect(void);
					virtual void Select(u8 item);
					virtual void Deselect(u8 item);
				protected:
				private:
			};
			class Resolution : public GUIRadioCheckBox {
				public:
					Resolution(void);
					virtual u8 QuerySelect(void);
					virtual void Select(u8 item);
					virtual void Deselect(u8 item);
				protected:
				private:
			};
			class RegisterFormat : public GUIRadioCheckBox {
				public:
					RegisterFormat(void);
					virtual u8 QuerySelect(void);
					virtual void Select(u8 item);
					virtual void Deselect(u8 item);
				protected:
				private:
			};
			class MemoryFormat : public GUIRadioCheckBox {
				public:
					MemoryFormat(void);
					virtual u8 QuerySelect(void);
					virtual void Select(u8 item);
					virtual void Deselect(u8 item);
				protected:
				private:
			};
			class AppModeSwitch : public GUICheckBox {
				public:
					AppModeSwitch(void);
					virtual Bool QuerySelect(void);
					virtual void Select(void);
					virtual void Deselect(void);
				protected:
				private:
			};
			class DebugLog : public GUICheckBox {
				public:
					DebugLog(void);
					virtual Bool QuerySelect(void);
					virtual void Select(void);
					virtual void Deselect(void);
				protected:
				private:
			};
			class AppModeWindow : public GUICheckBox {
				public:
					AppModeWindow(void);
					virtual Bool QuerySelect(void);
					virtual void Select(void);
					virtual void Deselect(void);
				protected:
				private:
			};
			class SkipStartup : public GUICheckBox {
				public:
					SkipStartup(void);
					virtual Bool QuerySelect(void);
					virtual void Select(void);
					virtual void Deselect(void);
				protected:
				private:
			};
			class UseThumbSyntax : public GUICheckBox {
				public:
					UseThumbSyntax(void);
					virtual Bool QuerySelect(void);
					virtual void Select(void);
					virtual void Deselect(void);
				protected:
				private:
			};
			class UseMnemonic : public GUICheckBox {
				public:
					UseMnemonic(void);
					virtual Bool QuerySelect(void);
					virtual void Select(void);
					virtual void Deselect(void);
				protected:
				private:
			};
			class SoundEnable : public GUICheckBox {
				public:
					SoundEnable(void);
					virtual Bool QuerySelect(void);
					virtual void Select(void);
					virtual void Deselect(void);
				protected:
				private:
			};
			#if !defined(RELEASE_BUILD)
				Language m_language;
			#endif
			LineCount m_lineCount;
			RegisterFormat m_registerFormat;
			MemoryFormat m_memoryFormat;
			AppModeSwitch m_runAppModeSwitch;
			DebugLog m_debugLog;
			AppModeWindow m_runAppWindow;
			SkipStartup m_skipStartup;
			UseThumbSyntax m_useThumbSyntax;
			UseMnemonic m_useMnemonic;
			#if !defined(RELEASE_BUILD)
				SoundEnable m_soundEnable;
			#endif
			Resolution m_resolution;
			u16 m_curRow, m_curCol, m_maxWidth;
	};

	/////////////////////////////////////////////////////////////////////////
	//	This panel contains warning messages for the user to heed.
	class UserWarningPanel : public IListTextPanel {
		public:
			void AddWarning(const IString & f);
		protected:
		private:
	};

	/////////////////////////////////////////////////////////////////////////
	//	This window holds the largest blocks of text.
	class CodeWindow : public Window {
		public:
			enum {
				TAB_DISASSEMBLY			= 0,
				TAB_LOAD,
				TAB_OPTIONS,
				TAB_WARNINGS,
				TAB_USER,
				TAB_HELP,
			};

			CodeWindow(void);
			virtual void SetTab(u8 tab);

			// The various tracing options.
			void SingleStep(void);
			void Goto(u32 line);

			// Determine the current cursor IP.  If the IP is invalid, we
			// return 0xffffffff.
			u32 GetCurrentCursorIP(void);

			void AddWarning(const IString & f);
			void AddUserMessage(const FString & f);

			void GenerateContent(u8 panel);
		protected:
		private:
			DisassemblyPanel m_disassembly;
			BrowserPanel m_load;
			OptionsPanel m_options;
			UserWarningPanel m_warnings;
			FListTextPanel m_user;
			FileTextPanel m_help;
	};

	#include "window1.ipp"

	extern Sticky < u8 > g_cursorDisplayOption;
	extern CodeWindow * g_codeWindow;
	extern void WarnUser(const IString & f);

#endif
