/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	This is our second window, the data window.

****************************************************************************/

#ifndef WINDOW2_H
	#define WINDOW2_H

	// These are the available memory formats.  Note:  I'm allowing fewer
	// formats for the register panel.
	enum {
		MBYTE		= 0,
		MHALFWORD,
		MWORD,

		MSIGNED		= 4,
		MDECIMAL	= 8,
		MBINARY		= 16,
		MTEXT		= 32,
		MWTEXT,				// Wide-character text.
		MFLOAT
	};

	/////////////////////////////////////////////////////////////////////////
	//	Generate content for the memory panel.
	class MemoryPanel : public TextPanel {
		public:
			virtual void CursorUp(void);
			virtual void CursorDown(void);
			virtual void PageUp(void);
			virtual void PageDown(void);

			virtual void Goto(u32 line);

			virtual void SetDims(u16 x, u16 y, u16 w, u16 h);
			virtual void GenerateContent(void);
		protected:
		private:
			u32 m_address;
			u32 m_bottom;
			u16 m_step;		// The number of bytes, dwords, etc. per line.
	};

	/////////////////////////////////////////////////////////////////////////
	//	This panel displays all of the breakpoints the user has entered.  We
	// will be adding a variety of interfaces for entering breakpoints.
	class BreakpointPanel : public TextPanel {
		public:
			BreakpointPanel(void);
			void AddBreakpoint(u32 address, u32 range, u8 conditions);

			virtual void RClick(u16 x, u16 y);
			virtual void Click(u16 x, u16 y);
			virtual void SetDims(u16 x, u16 y, u16 w, u16 h);

			virtual void CursorUp(void);
			virtual void CursorDown(void);
			virtual void PageUp(void);
			virtual void PageDown(void);

			virtual void GenerateContent(void);
			virtual void Redraw(void);
		protected:
		private:
			class BreakCheck : public GUICheckBox {
				public:
					void SetMask(u8 mask);
					void SetBP(MType bp);
					virtual Bool QuerySelect(void);
					virtual void Select(void);
					virtual void Deselect(void);
				protected:
				private:
					MType m_bp;
					u8 m_mask;
			};
			u32 m_currentBreakpoint;
			Vec < Array < BreakCheck > , 3 > m_checks;
	};

	/////////////////////////////////////////////////////////////////////////
	//	The OAM panel lets you browse the sprites.
	class OAMPanel : public Panel {
		public:
			OAMPanel(void);

			virtual void GenerateContent(void);
			virtual void Redraw(void);
			virtual void CursorUp(void);
			virtual void CursorDown(void);
			virtual void PageUp(void);
			virtual void PageDown(void);
		protected:
		private:
			u8 m_currentSprite;
	};

	/////////////////////////////////////////////////////////////////////////
	//	This window holds the largest blocks of text.
	class DataWindow : public Window {
		public:
			enum {
				#if !defined(RELEASE_BUILD)
					TAB_WATCHES,
				#endif
				TAB_MEMORY,
				TAB_BREAKPOINTS,
				TAB_OAM,
			};

			DataWindow(void);
			virtual void SetTab(u8 tab);
			void SingleStep(void);
			void Goto(u32 u);
			void GenerateContent(u8 panel);
			void AddBreakpoint(u32 address, u32 range, u8 conditions);
		protected:
		private:
			#if !defined(RELEASE_BUILD)
				DummyPanel m_watches;
			#endif
			MemoryPanel m_memory;
			BreakpointPanel m_breakpoints;
			OAMPanel m_oam;
	};

	//  This tells us how much each component of a memory window is, given
	// the format.
	extern u8 MemoryGetWidth(u8 format);
	extern void MemoryTranslate(FString & f, u32 value, u8 format);

	#include "window2.ipp"

	extern Sticky < u8 > g_memoryFormat;
	extern DataWindow * g_dataWindow;

#endif
