/****************************************************************************

	Written by John Marco Panettiere for John Marco Panettiere.
	Copyright 1998, 1999, 2000 by John Marco Panettiere.
	All rights reserved.

	This is the final window of my GUI:  the peripheral window.

****************************************************************************/

#include "gba.h"

static TString s_app("App");
static TString s_registers("Registers");
static TString s_dma("DMA");

Sticky < u8 > g_registerFormat("Register Format", MWORD);

/////////////////////////////////////////////////////////////////////////////
//  A goto request in the register window will relocate R15 to that location.
void RegisterPanel::Goto(u32 u) {
	g_gba.GetCPU().SetRegister(15, u);
	// HEREHEREHERE we need to update the disassembly window and the source
	// window.
	GenerateContent();
}

/////////////////////////////////////////////////////////////////////////////
//  Generate content for the register window.
void RegisterPanel::GenerateContent(void) {
	MType n, k, m;
	u16 fw = TextGetFontWidth(g_font);
	u8 cWidth = MemoryGetWidth(*g_registerFormat);
	u16 tWidth = (cWidth + 4) * fw;
	u8 regsPerLine = (m_rect.w + 2) / (tWidth + 2);

	m_content[0] = TEXT_COLOR_ORANGE;
	for (n=0, k=0, m=0; m<16; ++m, ++k) {
		if (k == regsPerLine) {
			k = 0;
			++n;
			if (n == m_content.Count()) {
				break;
			}
			m_content[n] = TEXT_COLOR_ORANGE;
		}
		m_content[n] += "R";
		if (m > 9) {
			m_content[n] += "1";
			m_content[n] += (char)('0' + m - 10);
		} else {
			m_content[n] += "0";
			m_content[n] += (char)('0' + m);
		}
		m_content[n] += ":";
		if (g_gba.GetCPU().GetRegister(m) != m_oldValues[m]) {
			m_content[n] += (wchar_t)TEXT_COLOR_RED;
		} else {
			m_content[n] += (wchar_t)TEXT_COLOR_WHITE;
		}
		MemoryTranslate(m_content[n],
						g_gba.GetCPU().GetRegister(m),
						*g_registerFormat);
		m_oldValues[m] = g_gba.GetCPU().GetRegister(m);
		m_content[n] += (wchar_t)TEXT_COLOR_ORANGE;
	}
	++n;
	m_content[n] = TEXT_COLOR_ORANGE;
	m_content[n] += "CPSR:";
	if (g_gba.GetCPU().GetRegister(CPSR) != m_oldValues[16]) {
		m_content[n] += (wchar_t)TEXT_COLOR_RED;
	} else {
		m_content[n] += (wchar_t)TEXT_COLOR_WHITE;
	}
	MemoryTranslate(m_content[n], g_gba.GetCPU().GetRegister(CPSR), *g_registerFormat);
	m_oldValues[16] = g_gba.GetCPU().GetRegister(CPSR);
	m_content[n] += (wchar_t)TEXT_COLOR_ORANGE;
	
	++n;
	m_content[n] = TEXT_COLOR_ORANGE;
	m_content[n] += "SPSR:";
	if (g_gba.GetCPU().GetRegister(SPSR) != m_oldValues[17]) {
		m_content[n] += (wchar_t)TEXT_COLOR_RED;
	} else {
		m_content[n] += (wchar_t)TEXT_COLOR_WHITE;
	}
	MemoryTranslate(m_content[n], g_gba.GetCPU().GetRegister(SPSR), *g_registerFormat);
	m_oldValues[17] = g_gba.GetCPU().GetRegister(SPSR);
	m_content[n] += (wchar_t)TEXT_COLOR_ORANGE;
	
	for (++n; n<m_content.Count(); ++n) {
		m_content[n].Destroy();
	}
	MakeDirty();
}

static TString s_dmaSource0("DMA 0 Src:   ");
static TString s_dmaDest0("DMA 0 Dst:   ");
static TString s_dmaControl0("DMA 0 Ctrl:  ");
static TString s_dmaSource1("DMA 1 Src:   ");
static TString s_dmaDest1("DMA 1 Dst:   ");
static TString s_dmaControl1("DMA 1 Ctrl:  ");
static TString s_dmaSource2("DMA 2 Src:   ");
static TString s_dmaDest2("DMA 2 Dst:   ");
static TString s_dmaControl2("DMA 2 Ctrl:  ");
static TString s_dmaSource3("DMA 3 Src:   ");
static TString s_dmaDest3("DMA 3 Dst:   ");
static TString s_dmaControl3("DMA 3 Ctrl:  ");

/////////////////////////////////////////////////////////////////////////////
//  Generate content for the DMA panel.
void DMAPanel::GenerateContent(void) {
	u16 u;
	u8 v, w;
	static TString * s_strings[] = {
		&s_dmaSource0,
		&s_dmaDest0,
		&s_dmaControl0,
		&s_dmaSource1,
		&s_dmaDest1,
		&s_dmaControl1,
		&s_dmaSource2,
		&s_dmaDest2,
		&s_dmaControl2,
		&s_dmaSource3,
		&s_dmaDest3,
		&s_dmaControl3,
	};

	TextSetTab(0, m_rect.x + TextGetFontWidth(g_font) * 14);
	for (v=w=0; v<12; v+=3) {
		m_content[w] = TEXT_COLOR_ORANGE;
		m_content[w] += *(s_strings[v + 0]);
		m_content[w] += '\t';
		ConvertFromHex(m_content[w], g_gba.GetMemory().GetD(DMA0_SOURCE + v * 4));

		++w;
		m_content[w] = TEXT_COLOR_ORANGE;
		m_content[w] += *(s_strings[v + 1]);
		m_content[w] += '\t';
		ConvertFromHex(m_content[w], g_gba.GetMemory().GetD(DMA0_DEST + v * 4));

		++w;
		m_content[w] = TEXT_COLOR_ORANGE;
		m_content[w] += *(s_strings[v + 2]);
		m_content[w] += '\t';
		u = g_gba.GetMemory().GetW(DMA0_CONTROL + v * 4);
		if (u & DMA_ENABLE) {
			if (g_gba.GetDMA().Busy()) {
				m_content[w] += (wchar_t)TEXT_COLOR_GREEN;
			} else {
				m_content[w] += (wchar_t)TEXT_COLOR_YELLOW;
			}
		} else {
			m_content[w] += (wchar_t)TEXT_COLOR_RED;
		}
		if (u & DMA_IE) {
			m_content[w] += "IE ";
		}
		switch(u & DMA_START_MASK) {
			case DMA_START_IMM:
				m_content[w] += "IM ";
				break;
			case DMA_START_VBL:
				m_content[w] += "VB ";
				break;
			case DMA_START_HBL:
				m_content[w] += "HB ";
				break;
			default:
				m_content[w] += "?? ";
		}
		if (u & DMA_XFER32) {
			m_content[w] += "[32] ";
		} else {
			m_content[w] += "[16] ";
		}
		if (u & DMA_REPEAT_ON) {
			m_content[w] += "RP ";
		}
		switch((u >> 7) & 3) {
			case 0:
				m_content[w] += "S+ ";
				break;
			case 1:
				m_content[w] += "S- ";
				break;
			case 2:
			case 3:
				m_content[w] += "S= ";
				break;
		}
		switch((u >> 5) & 3) {
			case 0:
			case 3:
				m_content[w] += "D+";
				break;
			case 1:
				m_content[w] += "D-";
				break;
			case 2:
				m_content[w] += "D=";
				break;
		}
		++w;

//		m_content[w].Destroy();
//		++w;
	}
	MakeDirty();
}

/////////////////////////////////////////////////////////////////////////////
//  Build the app window.
AppWindow::AppWindow(void) {
	SetXYWH(g_monitor.GetWidth() - EMU_WINDOW_WIDTH,
			g_monitor.GetHeight() - EMU_WINDOW_HEIGHT - STATUS_BAR_HEIGHT,
			EMU_WINDOW_WIDTH,
			EMU_WINDOW_HEIGHT);
	AddPanel(s_app, &m_app);
	AddPanel(s_registers, &m_registers);
	AddPanel(s_dma, &m_dma);
}

/////////////////////////////////////////////////////////////////////////////
//  Specify the active tab.
void AppWindow::SetTab(u8 tab) {
	switch(tab) {
		case TAB_APP:
			m_app.GenerateContent();
			break;
		case TAB_REGISTERS:
			m_registers.GenerateContent();
			break;
		case TAB_DMA:
			m_dma.GenerateContent();
			break;
	}
	Window::SetTab(tab);
}

/////////////////////////////////////////////////////////////////////////////
//  Respond to the progress of the program.
void AppWindow::SingleStep(void) {
	switch(m_selection) {
		case TAB_REGISTERS:
			m_registers.GenerateContent();
			break;
		case TAB_DMA:
			m_dma.GenerateContent();
			break;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Respond to a goto command.  For now, we do nothing.
void AppWindow::Goto(u32 u) {
	switch(m_selection) {
		case TAB_REGISTERS:
			m_registers.Goto(u);
			break;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Instruct the specified panel to generate content.
void AppWindow::GenerateContent(u8 panel) {
	switch(panel) {
		case TAB_APP:
			m_app.GenerateContent();
			break;
		case TAB_REGISTERS:
			m_registers.GenerateContent();
			break;
		case TAB_DMA:
			m_dma.GenerateContent();
			break;
	}
}

/////////////////////////////////////////////////////////////////////////////
//  Specify the dimensions of the app window.
void AppPanel::SetDims(u16 x, u16 y, u16 w, u16 h) {
	m_bitmap.Allocate(w, h, __FILE__, __LINE__);
	m_bitmap.Clear(0);
	Panel::SetDims(x, y, w, h);
}

/////////////////////////////////////////////////////////////////////////////
//  Redraw the app panel.
void AppPanel::Redraw(void) {
	RenderOptions ro;

	m_bitmap.Render(g_monitor,
					m_rect.x,
					m_rect.y,
					ro);
	Panel::Redraw();
	CleanFrame();
}

/////////////////////////////////////////////////////////////////////////////
//  Move the cursor up.  Presently, this does nothing.
void AppPanel::CursorUp(void) {
}

/////////////////////////////////////////////////////////////////////////////
//  Move the cursor down (do nothing).
void AppPanel::CursorDown(void) {
}

/////////////////////////////////////////////////////////////////////////////
//  Page-up.
void AppPanel::PageUp(void) {
}

/////////////////////////////////////////////////////////////////////////////
//  Page-down.
void AppPanel::PageDown(void) {
}

/////////////////////////////////////////////////////////////////////////////
//  Generate content for the panel.
void AppPanel::GenerateContent(void) {
	MakeDirty();
}

